import random

from   OpenGL.GL import *
from   OpenGL.GLU import *

import Texture
import config

class SourceScroll:
   def __init__(self, source_file, font_name, font_size, color, min_lines, max_lines, total_blocks, add_blocks, speed):
      self.color = color
      self.add_blocks = add_blocks
      self.speed = speed

      self.source = Texture.SourceCode(source_file, font_name, font_size, min_lines, max_lines, total_blocks)
      self.src_texts = []
      self.src_pos = []

   def getSpeed(self):
      return self.speed

   def setSpeed(self, speed):
      self.speed = speed

   def update(self, elapsed_ms):
      dt = elapsed_ms / 1000.0

      maxy = 0.5
      for text, pos in zip(self.src_texts, self.src_pos):
         pos[1] -= self.speed * dt
         maxy = max(maxy, pos[1] + text.getMaxY())
         if pos[1] + text.getMaxY() < -0.5:
            self.src_texts.remove(text)
            self.src_pos.remove(pos)
      
      if maxy < 1.0:
         new_texts = self.source.getBlocks(self.add_blocks)
         new_pos = []
         for text in new_texts:
            miny = text.getMinY()
            new_pos.append( [random.uniform(-0.5, 0.5), random.uniform(0.8 - miny, 1.0 - miny)] )

         self.src_texts.extend(new_texts)
         self.src_pos.extend(new_pos)

   def draw(self):
      glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT)

      glDisable(GL_TEXTURE_2D)
      glDisable(GL_LIGHTING)

      glColor4f(*self.color)
      for text, (x, y) in zip(self.src_texts, self.src_pos):
         glPushMatrix()
         glLoadIdentity()
         glTranslatef(x, y, 0.0)
         text.draw()
         glPopMatrix()

      glPopAttrib()

class WinLose:
   def __init__(self):
      self.wintext = Texture.Text(
         "All memory freed! Nice work :)",
         config.MENU_FONT,
         50
      )
      
      self.losetext = Texture.Text(
         "All memory leaked :(",
         config.MENU_FONT,
         50
      )

      self.period = 1.0

      self.on = False
      self.sumt = 0.0
      self.alpha = 1.0
   
   def setOn(self, won = True):
      self.on = True
      self.won = won

      if won: self.txt = self.wintext
      else: self.txt = self.losetext

   def update(self, dt):
      if self.on:
         self.sumt += dt
         
         if self.sumt >= 2 * self.period:
            self.sumt -= 2 * self.period

         if self.sumt <= self.period:
            self.alpha = 0.5 + 0.5 * self.sumt / self.period
         else:
            self.alpha = 1.0 - 0.5 * (self.sumt - self.period) / self.period

   def draw(self):
      if not self.on:
         return
      
      glColor4f(1.0, 1.0, 1.0, self.alpha)
      self.txt.draw(Texture.Text.HALIGN_CENTER, Texture.Text.VALIGN_CENTER)
      

class ScoreCounter:
   def __init__(self, score_text, text_font_name, text_font_size, score_font_name, score_font_size, speed):
      self.stext = Texture.Text(score_text, text_font_name, text_font_size)
      self.mtext = Texture.MonoText(score_font_name, score_font_size)
      self.speed = speed
      self.score = 0
      self.dest_score = 0
      self.act_score = 0.0

   def addScore(self, score):
      self.dest_score += score

   def update(self, elapsed_ms):
      if self.score != self.dest_score:
         dt = elapsed_ms / 1000.0
         if self.score < self.dest_score:
            self.act_score += self.speed * dt
            if self.act_score > self.dest_score:
               self.act_score = self.dest_score
         else:
            self.act_score -= self.speed * dt
            if self.act_score < self.dest_score:
               self.act_score = self.dest_score

   def draw(self):
      self.stext.draw(Texture.Text.HALIGN_LEFT, Texture.Text.VALIGN_CENTER)
      w, h = self.stext.getSize()
      glPushMatrix()
      #glTranslatef(w, -0.004, 0.0)
      glTranslatef(w, 0.0, 0.0)
      self.mtext.draw("%d" % int(self.act_score), h, Texture.MonoText.HALIGN_LEFT, Texture.MonoText.VALIGN_CENTER)
      glPopMatrix()

class MemoryCounter:
   def __init__(self, size, font_name, font_size, max_memory, speed):
      self.size = size
      self.text = Texture.MonoText(font_name, font_size)
      self.speed = speed
      self.memory = 0
      self.max_memory = max_memory
      self.dest_memory = 0

   def addMemory(self, mem):
      self.dest_memory = max(0, min(self.max_memory, self.dest_memory + mem))

   def update(self, elapsed_ms):
      if self.memory != self.dest_memory:
         dt = elapsed_ms / 1000.0
         if self.memory < self.dest_memory:
            self.memory = min(self.dest_memory, self.memory + self.speed * dt)
         else:
            self.memory = max(self.dest_memory, self.memory - self.speed * dt)

   def draw(self):
      glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT)
      glDisable(GL_TEXTURE_2D)
      glDisable(GL_LIGHTING)

      w, h = self.size
      memw = w * self.memory / self.max_memory
      dmemw = w * self.dest_memory / self.max_memory
      w2, h2 = w / 2.0, h / 2.0

      glColor4f(0.7, 0.0, 0.0, 1.0)
      glBegin(GL_QUADS)
      glVertex2f(-w2, h2);
      glVertex2f(-w2, -h2);
      glVertex2f(-w2 + memw, -h2);
      glVertex2f(-w2 + memw, h2);
      glEnd()
      
      glColor4f(0.7, 0.7, 0.0, 1.0)
      glBegin(GL_QUADS)
      glVertex2f(-w2 + memw, h2);
      glVertex2f(-w2 + memw, -h2);
      glVertex2f(-w2 + dmemw, -h2);
      glVertex2f(-w2 + dmemw, h2);
      glEnd()

      glColor3f(1.0, 1.0, 1.0)
      glBegin(GL_LINE_LOOP)
      glVertex2f(-w2, h2);
      glVertex2f(-w2, -h2);
      glVertex2f(w2, -h2);
      glVertex2f(w2, h2);
      glEnd()

      self.text.draw("%dk/%dk" % (int(self.memory), int(self.max_memory)), h * 0.5)

      glPopAttrib()
