import math

from   OpenGL.GL import *
from   OpenGL.GLU import *

import config

class Geometry:
   def __init__(self):
      self.vertices = []
      self.tri_faces = []
      self.quad_faces = []

      self.create()
      self._registerList()

   def _registerList(self):
      self.list_id = glGenLists(1)
      glNewList(self.list_id, GL_COMPILE)

      glPushAttrib(GL_ENABLE_BIT)

      glDisable(GL_TEXTURE_2D)
      glEnable(GL_NORMALIZE)

      if self.tri_faces:
         glEnable(GL_LIGHTING)
         glBegin(GL_TRIANGLES)
         for normal, iv in self.tri_faces:
            glNormal3f(*normal)
            for i in iv:
               glVertex3f(*self.vertices[i])
         glEnd()

         glDisable(GL_LIGHTING)
         for normal, iv in self.tri_faces:
            glBegin(GL_LINE_LOOP)
            for i in iv:
               glVertex3f(*self.vertices[i])
            glEnd()

      if self.quad_faces:
         glEnable(GL_LIGHTING)
         glBegin(GL_QUADS)
         for normal, iv in self.quad_faces:
            glNormal3f(*normal)
            for i in iv:
               glVertex3f(*self.vertices[i])
         glEnd()
         
         glDisable(GL_LIGHTING)
         for normal, iv in self.quad_faces:
            glBegin(GL_LINE_LOOP)
            for i in iv:
               glVertex3f(*self.vertices[i])
            glEnd()

      glDisable(GL_LIGHTING)
      glBegin(GL_POINTS)
      for v in self.vertices:
         glVertex3f(*v)
      glEnd()

      glPopAttrib()
      glEndList()

   def addFace(self, iv):
      a = [self.vertices[iv[1]][i] - self.vertices[iv[0]][i] for i in xrange(3)]
      b = [self.vertices[iv[-1]][i] - self.vertices[iv[0]][i] for i in xrange(3)]
      n = [
         a[1] * b[2] - a[2] * b[1],
         -(a[0] * b[2] - a[2] * b[0]),
         a[0] * b[1] - a[1] * b[0]
      ]
      c = self.vertices[iv[0]]

      if n[0] * c[0] + n[1] * c[1] + n[2] * c[2] < 0.0:
         iv.reverse()
         n = map(lambda x: -x, n)

      '''
      l = 0.0
      for c in n:
         l += c**2
      l = 1.0 / math.sqrt(l)

      n = map(lambda x: x*l, n)
      '''

      if len(iv) == 3:
         self.tri_faces.append((n, iv))
      else:
         self.quad_faces.append((n, iv))

   def create(self):
      pass

   def draw(self):
      glCallList(self.list_id)

class Tetrahedron(Geometry):
   def __init__(self):
      Geometry.__init__(self)

   def create(self):
      phiaa = -19.471220333
      phia = math.pi * phiaa / 180.0
      sphia = math.sin(phia)
      cphia = math.cos(phia)
      rad120 = math.pi * 120.0 / 180.0
      
      self.vertices.append((0.0, 0.0, 1.0))
      
      ang = 0.0
      for i in xrange(3):
         self.vertices.append((
            math.cos(ang) * cphia,
            math.sin(ang) * cphia,
            sphia
         ))
         ang += rad120

      self.addFace([0, 1, 2])
      self.addFace([0, 2, 3])
      self.addFace([0, 3, 1])
      self.addFace([2, 1, 3])

class Cube(Geometry):
   def __init__(self):
      Geometry.__init__(self)

   def create(self):
      phiaa = 35.264391
      phia = math.pi * phiaa / 180.0;
      cphia = math.cos(phia)
      sphia = math.sin(phia)

      phib = -phia;
      cphib = math.cos(phib)
      sphib = math.sin(phib)

      ang90 = math.pi * 90.0 / 180.0;

      ang = 0.0
      for i in xrange(4):
         self.vertices.append((
            math.cos(ang) * cphia,
            math.sin(ang) * cphia,
            sphia
         ))
         ang += ang90
         
      ang = 0.0
      for i in xrange(4):
         self.vertices.append((
            math.cos(ang) * cphib,
            math.sin(ang) * cphib,
            sphib
         ))
         ang += ang90

      self.addFace([0, 1, 2, 3])
      self.addFace([7, 6, 5, 4])
      self.addFace([4, 5, 1, 0])
      self.addFace([5, 6, 2, 1])
      self.addFace([6, 7, 3, 2])
      self.addFace([7, 4, 0, 3])

class Octahedron(Geometry):
   def __init__(self):
      Geometry.__init__(self)

   def create(self):
      phiaa = 0.0
      phia = math.pi * phiaa / 180.0
      cphia = math.cos(phia)
      sphia = math.sin(phia)
      ang90 = math.pi * 90.0 / 180.0

      self.vertices.append( (0.0, 0.0, 1.0) )

      ang = 0.0
      for i in xrange(4):
         self.vertices.append((
            math.cos(ang) * cphia,
            math.sin(ang) * cphia,
            sphia
         ))
         ang += ang90

      self.vertices.append( (0.0, 0.0, -1.0) )

      self.addFace([0, 1, 2])
      self.addFace([0, 2, 3])
      self.addFace([0, 3, 4])
      self.addFace([0, 4, 1])
      self.addFace([5, 1, 2])
      self.addFace([5, 2, 3])
      self.addFace([5, 3, 4])
      self.addFace([5, 4, 1])
'''
class Ring(Geometry):
   def __init__(self, width, segments):
      self.width = width
      self.segments = segments
      Geometry.__init__(self)

   def _registerList(self):
      self.list_id = glGenLists(1)
      glNewList(self.list_id, GL_COMPILE)

      glPushAttrib(GL_ENABLE_BIT)

      glDisable(GL_TEXTURE_2D)
      glEnable(GL_NORMALIZE)

      r = 1.0 - self.width

      ang20 = math.pi * 20.0 / 180.0

      glNormal3f(0.0, 0.0, 1.0)

      start_ang = 0.0
      for j in xrange(4):
         glBegin(GL_TRIANGLE_FAN)
         glVertex2f(r * math.cos(start_ang), r * math.sin(start_ang))
         for i in xrange(self.segments + 1):
            ang = start_ang - ang20 + 2.0 * ang20 * i / self.segments
            sang = math.sin(ang)
            cang = math.cos(ang)

            glVertex2f(cang, sang)
         glEnd()
         start_ang += math.pi / 2.0

      glPopAttrib()
      glEndList()
'''

class Ring(Geometry):
   def __init__(self, width, segments):
      self.width = width
      self.segments = segments
      Geometry.__init__(self)

   def _registerList(self):
      self.list_id = glGenLists(1)
      glNewList(self.list_id, GL_COMPILE)

      glPushAttrib(GL_ENABLE_BIT)

      glDisable(GL_TEXTURE_2D)
      glEnable(GL_NORMALIZE)

      ri = 1.0 - self.width / 2.0
      ro = 1.0 + self.width / 2.0

      step = 2.0 * math.pi / self.segments

      glNormal3f(0.0, 0.0, 1.0)

      for i in xrange(0, self.segments, 4):
         ang = 2.0 * math.pi * i / self.segments
         glBegin(GL_TRIANGLE_STRIP)
         glVertex2f(ri * math.cos(ang - step), ri * math.sin(ang - step))
         glVertex2f(ro * math.cos(ang), ro * math.sin(ang))
         glVertex2f(ri * math.cos(ang), ri * math.sin(ang))
         glVertex2f(ri * math.cos(ang + step), ri * math.sin(ang + step))
         glEnd()

      glPopAttrib()
      glEndList()

class GameObject:
   def __init__(self, name, radius, face_color, edge_color = (1.0, 1.0, 1.0, 1.0)):
      self.obj = config.geometry_manager[name]
      self.radius = radius
      self.face_color = face_color
      self.edge_color = edge_color
      self.rotation = [0.0, 0.0, 0.0]
      self.position = [0.0, 0.0, 0.0]

   def getPos(self):
      return self.position
 
   def setPos(self, pos):
      self.position = pos

   def getRot(self):
      return self.rotation

   def setRot(self, rot):
      self.rotation = rot

   def draw(self):
      glPushMatrix()
      glPushAttrib(GL_CURRENT_BIT)
      glTranslatef(*self.position)
      glRotate(self.rotation[0], 1.0, 0.0, 0.0)
      glRotate(self.rotation[1], 0.0, 1.0, 0.0)
      glRotate(self.rotation[2], 0.0, 0.0, 1.0)
      glScalef(self.radius, self.radius, self.radius)
      glMaterialfv(GL_FRONT, GL_DIFFUSE, self.face_color)
      glColor4f(*self.edge_color)
      self.obj.draw()
      glPopAttrib()
      glPopMatrix()

