import os

import pygame
from pygame.locals import *

import config

class ImageManager:
   """
   Image cache
   usage:
    pygame.Surface = config.image_manager["image_file"]

   """
   def __init__(self):
      self.images = {}
      
      if not config.__dict__.has_key("image_manager"):
         config.image_manager = self

   def cache(self, imglist):
      for i in imglist:
         self.load(i)
      
   def load(self, image):
      if not self.images.has_key(image):
         self.images[image] = pygame.image.load(os.path.join(config.IMAGE_PATH, image)).convert_alpha()
         
      return self.images[image]

   def __getitem__(self, image):
      if not self.images.has_key(image):
         self.load(image)
         
      return self.images[image]

class SoundManager:
   """
   Sound cache
   usage:
    pygame.Sound = config.image_manager["sound_file"]

   """

   def __init__(self):
      self.sounds = {}
      self.music = {}
    
      
      ###### PRELOADING ######
      preload = [
         "select.ogg",
         "collect.ogg",
         "invalid_chain.ogg",
         "pickup.ogg",
         "switch.ogg"
      ]
      for l in preload:
         self.load(l)
      #### 

      if not config.__dict__.has_key("sound_manager"):
         config.sound_manager = self

   def cache(self, sndlist):
      for i in sndlist:
         self.load(i)
      
   def load(self, sound):

      class Dummy:
         def play():
            pass

      try:
         if not self.sounds.has_key(sound):
            if pygame.mixer.get_init() is not None:
               self.sounds[sound] = pygame.mixer.Sound(os.path.join(config.SOUND_PATH, sound))
            else:
               self.souds[sound] = Dummy()
      except:
         print "[sound error, ignoring]"
         return Dummy()
         
      return self.sounds[sound]

   def playMusic(self, music):
      if pygame.mixer.get_init() is None:
         return
      try:
         if pygame.mixer.music.get_busy():
            pygame.mixer.music.fadeout(1500)
            pygame.mixer.music.queue(os.path.join(config.MUSIC_PATH, music))
         else:
            pygame.mixer.music.load(os.path.join(config.MUSIC_PATH, music))
            pygame.mixer.music.play()
      except:
         print "[music error, ignoring]"
         pass

   def __getitem__(self, sound):
      if not self.sounds.has_key(sound):
         self.load(sound)
         
      return self.sounds[sound]

class FontSizeManager:
   def __init__(self, font_file):
      self.font_file = font_file
      self.sizes = {}

   def __setitem__(self, key, value):
      self.sizes[key] = value

   def __getitem__(self, size):
      if not self.sizes.has_key(size):
         self.sizes[size] = pygame.font.Font(os.path.join(config.FONT_PATH, self.font_file), size)

      return self.sizes[size]

class FontManager:
   """
   Font cache
   usage:
    pygame.font.Font = config.font_manager["font_file"][font_size]

   """
   def __init__(self):
      self.fonts = {}
      
      if not config.__dict__.has_key("font_manager"):
         config.font_manager = self

   def cache(self, fontlist):
      for i in fontlist:
         self.load(*i)
      
   def load(self, font, size):
      if not self.fonts.has_key(font):
         self.fonts[font] = FontSizeManager()
         
      return self.fonts[font][size]

   def __getitem__(self, font):
      if not self.fonts.has_key(font):
         self.fonts[font] = FontSizeManager(font)
         
      return self.fonts[font]

class SourceManager:
   def __init__(self):
      self.sources = {}

      if not config.__dict__.has_key("source_manager"):
         config.source_manager = self

   def load(self, source):
      name, ext = os.path.splitext(source)
      if ext == ".c":
         sub = "c"
      elif ext ==".cpp":
         sub = "c++"
      elif ext == ".py":
         sub = "python"
      elif ext == ".asm":
         sub = "assembly"
            
      f = file(os.path.join(config.SOURCE_PATH, sub, source))
      lines = f.readlines()
      f.close()

      lines = filter(lambda x: x.lstrip(), lines)
      self.sources[source] = map(lambda x: x.rstrip().replace("\t", "   "), lines)

   def __getitem__(self, source):
      if not self.sources.has_key(source):
         self.load(source)

      return self.sources[source]

class GeometryManager:
   def __init__(self):
      self.geometries = {}
      
      if not config.__dict__.has_key("geometry_manager"):
         config.geometry_manager = self

   def add(self, name, geometry):
      self.geometries[name] = geometry

   def __getitem__(self, name):
      return self.geometries[name]

class MenuManager:
   """
   Menu cache
   usage:
    GrMenu = config.menu_manager["menu_name"]

   """
   def __init__(self, menu_file):
      f = file(menu_file)
      data = yaml.load(f)
      f.close()

      self.menus = {}
      for name, menu in data.iteritems():
         self.menus[name] = GrMenu(menu)

      if not config.__dict__.has_key("menu_manager"):
         config.menu_manager = self

   def __len__(self):
      return len(self.menus)

   def __getitem__(self, name):
      return self.menus[name]

