import pygame
from   pygame.locals import *

from   OpenGL.GL import *
from   OpenGL.GLU import *

import config
import Geometry, Texture, Component
from   Menu import *
from   Chain import *

import random as rand
import math

class SceneEventListener:
   def __init__(self):
      pass

   def onKeyEvent(self, event):
      pass

   def onKeyupEvent(self, event):
      pass

   def onMouseEvent(self, event):
      pass

class Scene:
   def __init__(self):
      self.quit = False
      self.next_scene = None

      self.key_event_listeners = []
      self.keyup_event_listeners = []
      self.mouse_event_listeners = []

   def registerConfig(self):
      config.act_scene = self

   def deregisterConfig(self):
      config.act_scene = None

   def addKeyEventListener(self, obj):
      self.key_event_listeners.append(obj)

   def removeKeyEventListener(self, obj):
      self.key_event_listeners.remove(obj)
   
   def addKeyupEventListener(self, obj):
      self.keyup_event_listeners.append(obj)

   def removeKeyupEventListener(self, obj):
      self.keyup_event_listeners.remove(obj)

   def addMouseEventListener(self, obj):
      self.mouse_event_listeners.append(obj)

   def removeMouseEventListener(self, obj):
      self.mouse_event_listeners.remove(obj)

   def getNextScene(self):
      return self.next_scene
   
   def setNextScene(self, next_scene):
      self.next_scene = next_scene

   def endLoop(self):
      self.quit = True

   def init(self):
      pass

   def uninit(self):
      pass

   def preRender(self, elapsed_ms):
      pass
   
   def update(self, elapsed_ms):
      pass

   def draw(self, elapsed_ms):
      pass

   def run(self):
      self.registerConfig()

      self.init()

      clock = pygame.time.Clock()

      pygame.event.clear()
      while not self.quit:
         elapsed_ms = clock.tick(config.FPS)

         for event in pygame.event.get():
            if event.type == KEYDOWN:
               for l in self.key_event_listeners:
                  l.onKeyEvent(event)

            elif event.type == KEYUP:
               for l in self.keyup_event_listeners:
                  l.onKeyUpEvent(event)
               
            elif event.type == MOUSEMOTION or event.type == MOUSEBUTTONDOWN:
               for l in self.mouse_event_listeners:
                  l.onMouseEvent(event)

         self.preRender(elapsed_ms)
         self.update(elapsed_ms)
         self.draw(elapsed_ms)

         pygame.display.flip()

      self.uninit()
     
      self.deregisterConfig()

      return self.next_scene

class MenuScene(Scene):
   def __init__(self):
      Scene.__init__(self)

      self.game_title = Texture.Text("Garbage Collector", config.TITLE_FONT, 90)
      self.game_title2 = Texture.Text("a PyWeek #7 entry", config.TITLE_FONT, 40)

      self.menu = Menu(["Python", "C", "C++", "Assembly", "Help", "Exit"], config.MENU_FONT, 60, (1.0, 1.0, 1.0, 0.8), (1.0, 1.0, 0.0, 0.5), -0.15, 0.8, 0.7)
      self.menu.setSelectCallback(self.onMenuSelect)

      self.background = Component.SourceScroll(rand.choice(config.SOURCE_ALL), config.SOURCE_FONT, 20, (0.1, 1.0, 0.1, 0.5), 10, 20, 20, 3, 0.1)

      self.gs = [CircleWanderChain((rand.uniform(-0.1, 0.1), rand.uniform(-0.1, 0.1)), rand.uniform(0.005, 0.009), rand.uniform(-1.0, 1.0)) for i in xrange(4)]
 
   def onKeyEvent(self, e):
      if e.key == K_ESCAPE:
         self.endLoop()
      elif e.key == K_F12:
         pygame.image.save(pygame.display.get_surface(), "screenshot.bmp")

   def onMenuSelect(self):
      menu_pt = self.menu.getSelected()
      if menu_pt < 4:
         self.setNextScene(GameScene(config.SOURCE_ALL[menu_pt], config.PARAMS[menu_pt]))
      elif menu_pt == 4:
         self.setNextScene(HelpScene())

      self.endLoop()

   def init(self):
      self.addKeyEventListener(self)
      self.addKeyEventListener(self.menu)
      self.addMouseEventListener(self.menu)
      
   def update(self, elapsed_ms):
      dt = elapsed_ms / 1000.0

      self.background.update(elapsed_ms)
      
      for g in self.gs:
         g.update(elapsed_ms)
         g.calcStep(dt)
         g.backProp(dt)
 
   def draw(self, elapsed_ms):
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

      glLoadIdentity()
      self.background.draw()
      
      glTranslatef(0.0, 0.5, 10.0)
      self.game_title.draw(Texture.Text.HALIGN_CENTER, Texture.Text.VALIGN_TOP)
      glTranslatef(0.0, -self.game_title.getSize()[1], 0.0)
      self.game_title2.draw(Texture.Text.HALIGN_CENTER, Texture.Text.VALIGN_TOP)

      glLoadIdentity()
      glTranslatef(0.0, 0.0, 20.0)
      glColor3f(1.0, 1.0, 1.0)
      for g in self.gs:
         g.draw()
         
      '''
      glTranslatef(0.0, 0.5, 10.0)
      self.game_title.draw(Texture.Text.HALIGN_CENTER, Texture.Text.VALIGN_TOP)
      glTranslatef(0.0, -0.17, 10.0)
      self.game_title2.draw(Texture.Text.HALIGN_CENTER, Texture.Text.VALIGN_TOP)
      glTranslatef(0.0, -0.33, 0.0)
      '''

      glTranslatef(0.0, 0.0, 10.0)
      self.menu.draw()

class HelpScene(Scene):
   def __init__(self):
      Scene.__init__(self)

      self.help_title = Texture.Text("Help", "256bytes.ttf", 90)

      font = config.SOURCE_FONT
      size = 20

      self.legend = [
         [
            Geometry.GameObject("tetra", 0.04, (0.0, 0.0, 1.0, 1.0), (0.0, 1.0, 0.0, 1.0)),
            Texture.MultilineText("Head of the garbage list. You control it with the mouse\nrest of the list will follow.", font, size, 0)
         ],

         [
            Geometry.GameObject("cube", 0.04, (0.0, 1.0, 0.0, 1.0)),
            Texture.MultilineText("20kBytes of unreferenced memory.", font, size, 0)
         ],
         
         [
            Geometry.GameObject("octa", 0.04, (0.0, 1.0, 0.0, 1.0)),
            Texture.MultilineText("40kBytes of unreferenced memory.", font, size, 0)
         ],

         [
            Geometry.GameObject("cube", 0.04, (1.0, 0.0, 0.0, 1.0)),
            Texture.MultilineText("80kBytes of unreferenced memory.", font, size, 0)
         ],
         
         [
            Geometry.GameObject("octa", 0.04, (1.0, 0.0, 0.0, 1.0)),
            Texture.MultilineText("160kBytes of unreferenced memory.", font, size, 0)
         ],
         
         [
            Geometry.GameObject("octa", 0.04, (0.3, 0.3, 0.3, 0.8), (0.0, 0.0, 0.0, 0.8)),
            Texture.MultilineText("A bug in the program. Catchig it will break your garbage\nchain and cause memory leak.", font, size, 0)
         ],


         [
            Geometry.GameObject("ring", 0.04, (1.0, 1.0, 0.0, 1.0)),
            Texture.MultilineText("Collecting point in program flow.\nYou can free collected objects there.", font, size, 0)
         ],
      ]

      n = len(self.legend) + 1
      self.pos = []
      for i in xrange(1, n):
         self.pos.append(0.5 - float(i) / n)

      for obj, text in self.legend:
         obj.rot_speed = [rand.uniform(-20.0, 20.0) for j in xrange(3)]

      self.legend[-1][0].rot_speed[:2] = [0.0, 0.0]

   def onKeyEvent(self, e):
      if e.key == K_ESCAPE:
         self.setNextScene(MenuScene())
         self.endLoop()
      elif e.key == K_F12:
         pygame.image.save(pygame.display.get_surface(), "screenshot.bmp")

   def init(self):
      self.addKeyEventListener(self)
      
   def update(self, elapsed_ms):
      dt = elapsed_ms / 1000.0
      
      for obj, text in self.legend:
         rot = obj.getRot()
         for i in xrange(3):
            rot[i] += obj.rot_speed[i] * dt

   def draw(self, elapsed_ms):
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

      glLoadIdentity()

      i = 0
      for obj, text in self.legend:
         glLoadIdentity()
         glTranslatef(config.SCREEN_LEFT + 0.1, self.pos[i], 0.0)
         obj.draw()
         glTranslatef(0.1, 0.0, 0.0)
         text.draw(Texture.Text.HALIGN_LEFT)
         i += 1

class GameScene(Scene):
   def __init__(self, source_file, params):
      Scene.__init__(self)

      self.background = Component.SourceScroll(source_file, config.SOURCE_FONT, 20, (0.1, 1.0, 0.1, 0.5), 10, 20, 20, 3, 0.1)

      self.score_counter = Component.ScoreCounter("Score: ", config.MENU_FONT, 30, config.MENU_FONT, 16, 257)
      
      max_mem, start_mem, (self.ringMin, self.ringMax), (self.genMin, self.genMax), self.badChainPoss, self.chainSpeedMul, self.levelCtrlHardness = params

      self.memory_counter = Component.MemoryCounter((0.4, 0.05), config.MENU_FONT, 16, max_mem, 451)
      self.memory_counter.addMemory(start_mem)

      self.winlose = Component.WinLose()

      self.ring = None      

      self.ce1 = Chain()
      self.ce1.initHead((-0.3, 0.2))
      #self.ce1.addElems(0)  

      self.gs = []
      #self.gs = [WanderChain() for i in xrange(4)]
      #self.gs = [CircleWanderChain((rand.uniform(-0.1, 0.1), rand.uniform(-0.1, 0.1)), rand.uniform(0.005, 0.009), rand.uniform(-1.0, 1.0)) for i in xrange(4)]
      
      self.ring = None
      self.nextRing = rand.uniform(self.ringMin, self.ringMax)
      self.nextGen = rand.uniform(self.genMin, self.genMax)

      self.leaverCE = None

      self.sumt = 0.0

      self.over = False
 
   def genRing(self):
      self.ring = Geometry.GameObject("ring", 0.05, (1.0, 1.0, 0.0, 0.5))
      self.ring_x = rand.uniform(-0.5, 0.5)
      self.ring_y = 0.6
      self.ring_ang = 0.0

   def onKeyEvent(self, e):
      if e.key == K_ESCAPE:
         self.setNextScene(MenuScene())
         self.endLoop()
      elif e.key == K_F12:
         pygame.image.save(pygame.display.get_surface(), "screenshot.bmp")

   def onMouseEvent(self, e):
      if e.type == MOUSEMOTION:
         # Add force to ce1
         x,y = map(lambda x:x/float(config.VIDEO_MODE[0]), e.rel)
         self.ce1.addRel(x*2,-y*2)
   
   def init(self):
      self.addKeyEventListener(self)
      self.addMouseEventListener(self)

      # Capture mouse
      pygame.mouse.set_visible(False)
      pygame.event.set_grab(True)
      pygame.mouse.set_pos(config.VIDEO_MODE[0] / 2, config.VIDEO_MODE[1] / 2)

   def uninit(self):
      # Give back mouse
      pygame.mouse.set_visible(True)
      pygame.event.set_grab(False)
      pygame.mouse.set_pos(config.VIDEO_MODE[0] / 2, config.VIDEO_MODE[1] / 2)
   
   def preRender(self, elapsed_msesc):
      pass
   
   def update(self, elapsed_ms):
      dt = elapsed_ms / 1000.0
      self.sumt += dt

      #######
      self.nextGen -= dt
      self.nextRing -= dt

      if len(self.gs) == 0:
         self.nextGen = min(self.nextGen, 1.0)

      if self.ring is None:
         if self.nextRing <= 0.0:
            self.genRing()
      
      if self.nextGen <= 0.0:
         self.nextGen = rand.uniform(self.genMin, self.genMax)

         # gen new chain
         c = WanderChain(self.badChainPoss, self.chainSpeedMul)
         c.setEdgeCol((0.0, 0.0, 0.0, 1.0))
         self.gs.append(c)
      
      ######

      self.ce1.calcStep(dt)
      self.ce1.backProp(dt)

      self.background.update(elapsed_ms)
      if self.ring:
         self.ring_ang += 30.0 * dt
         self.ring_y -= 0.1 * dt

         if self.ring_y <= -0.6:
            self.ring = None
            self.nextRing = rand.uniform(self.ringMin, self.ringMax)


      if not self.over and self.ring and self.ce1.isHeadColl(None, (self.ring_x, self.ring_y)):
         
         d_mem = - self.ce1.getMem()

         d_score = -d_mem

         try:
            self.leaverCE = self.ce1.getMainElem(1)
            self.leaverCE.tx = 0.0
            self.leaverCE.ty = -15.0
         except:
            # no child 
            pass

         self.ce1.detach()

         self.memory_counter.addMemory( d_mem )
         self.score_counter.addScore( d_score )
         self.ring = None
         self.nextRing = rand.uniform(self.ringMin, self.ringMax)
               
         config.sound_manager["collect.ogg"].play()

      self.score_counter.update(elapsed_ms)
      self.memory_counter.update(elapsed_ms)

      
      rem = []

      for g in self.gs:
         g.update(dt)
         g.calcStep(dt)
         g.backProp(dt)

         # picked up?
         if not self.over and not g.dying and self.ce1.isHeadColl(g):
            if g.size < 0.95:
               # too small yet
               continue

            if g.bad:
               config.sound_manager["invalid_chain.ogg"].play()
               print 'BAD CHAIN, GC ERROR'
               
               d_mem = g.getMem()
               self.memory_counter.addMemory( d_mem )

               rem.append(g)
            else:
               config.sound_manager["pickup.ogg"].play()
               self.ce1.addChain(g)
               rem.append(g)
               g.setEdgeCol((1.0, 1.0, 1.0, 1.0))

               g.head.geom.radius = 0.03

         # out of scene?

         def getScreenDist(x,y):
            if x < config.SCREEN_LEFT:
               dx = config.SCREEN_LEFT - x
            elif x > config.SCREEN_RIGHT:
               dx = x - config.SCREEN_LEFT
            else:
               dx = 0.0
 
            if y < -0.5:
               dy = -0.5 - y 
            elif y > 0.5:
               dy = y - 0.5
            else:
               dy = 0.0

            return math.sqrt(dx*dx + dy*dy)

         if not g.dying:

            dist = getScreenDist(g.head.px, g.head.py)
               
            maxdist = 0.1
            if g.circ: maxdist = 0.2

            if dist > maxdist:
               # out of gc
               g.markDead()
    
         # Gather dead ones
         if not self.over and g.dying and g.dead < 0.0:
            rem.append(g)

            if not g.bad:
               # Memory leak
               d_mem = g.getMem()
               self.memory_counter.addMemory( d_mem )

      for r in rem:
         self.gs.remove(r)

      if self.leaverCE is not None:
         self.leaverCE.calcStep(dt)
         self.leaverCE.backProp(dt)

      # win / lose condition 
      if self.memory_counter.memory == 0:
         self.winlose.setOn(True)
         self.over = True
         self.won = True
      elif self.memory_counter.memory == self.memory_counter.max_memory:
         self.winlose.setOn(False)
         self.over = True
         self.won = False

      self.winlose.update(dt)
      
      # ctr speed
      l = len( self.ce1.getMainString() )
      self.ce1.head.ease_param = 1.0 - 0.2 / (l * self.levelCtrlHardness)
 
   def draw(self, elapsed_ms):
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

      glLoadIdentity()
      self.background.draw()
      
      if self.ring:
         glTranslatef(self.ring_x, self.ring_y, 0.0)
         glRotatef(self.ring_ang, 0.0, 0.0, 1.0)
         self.ring.draw()
      
      glLoadIdentity()
      glTranslatef(0.0, 0.0, 10.0)
      self.ce1.draw()
      glColor3f(0.0, 0.0, 0.0)
      for g in self.gs:
         g.draw()
     
      if self.leaverCE:
         self.leaverCE.draw()

      
      glLoadIdentity()
      glColor3f(1.0, 1.0, 1.0)
      glTranslatef(config.SCREEN_LEFT + 0.05, -0.5 + 0.05, 0.0)
      self.score_counter.draw()
      
      
      glLoadIdentity()
      glColor3f(1.0, 1.0, 1.0)
      glTranslatef(config.SCREEN_RIGHT - 0.3, -0.5 + 0.05, 0.0)
      self.memory_counter.draw()

      glLoadIdentity()
      self.winlose.draw()

